package com.ruoyi.file.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.file.mapper.FileSequenceMapper;
import com.ruoyi.file.mapper.FilesMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@Service
public class FilesServiceImpl extends ServiceImpl<FilesMapper, Files> implements FilesService {
    @Value("${file.path}")
    private String filesUploadPath;//获取文件路径

    @Value("${file.domain}")
    private String filesUploadDomain;//文件域名

    @Value("${file.prefix}")
    private String filesPrefix;//文件前缀

    @Autowired
    private FileSequenceMapper fileSequenceMapper;


    @Override
    public FileUploadResponse upload(MultipartFile file) {

        // 文件夹路径名称
        String originalFilename = file.getOriginalFilename();

        //文件大小
        double size = file.getSize() / (1024.0);

        // 获取当前日期时间
        LocalDateTime now = LocalDateTime.now();
        String timestamp = now.format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
        int monthIncrement = getMonthIncrement(now.getYear(), now.getMonthValue()); // 获取每月递增值

        //文件类型
        String type = originalFilename.substring(originalFilename.lastIndexOf(".") + 1).toLowerCase();

        // 按照年月创建文件夹
        String yearMonth = now.format(DateTimeFormatter.ofPattern("yyyyMM"));
        // 新文件名格式：时间戳+每月递增值+文件类型
        String fileUuid = timestamp + "A" + String.format("%04d", monthIncrement) + "." + type;

        // 将相对路径转化为绝对路径
        String destPath = filesUploadPath + "/" + yearMonth;

        // 新的文件地址，绝对路径+新的文件名称
        File uploadFile = new File(destPath + "/" + fileUuid);

        // 判断配置的文件目录是否存在，若不存在则创建一个新的文件目录
        File parentFile = uploadFile.getParentFile();
        if (!parentFile.exists()) {
            parentFile.mkdirs();
        }

        try {
            // 存储文件到本地磁盘
            file.transferTo(uploadFile);

            // 设置文件url
            String url = filesUploadDomain + filesPrefix + "/" + yearMonth + "/" + fileUuid;

            // 将文件存储到数据库
            Files saveFile = new Files();
            saveFile.setFilesName(originalFilename);
            saveFile.setName(fileUuid);
            saveFile.setPath(destPath);
            saveFile.setType(type);
            saveFile.setSize(size); // （单位：KB）
            saveFile.setUrl(url);
            saveFile.setEnable(true);
            saveFile.setIsDelete(0);
            saveFile.setCreateTime(LocalDateTime.now());
            saveFile.setUpdateTime(null);
            // 保存操作
            save(saveFile);

            // 返回 FileUploadResponse 对象
            return new FileUploadResponse(fileUuid, url);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    private int getMonthIncrement(int year, int month) {
        // 查询当前月份的递增值
        FileSequence increment = fileSequenceMapper.getByYearAndMonth(year, month);
        if (increment == null) {
            // 如果没有记录，则初始化为1
            increment = new FileSequence();
            increment.setYear(year);
            increment.setMonth(month);
            increment.setValue(1);
            fileSequenceMapper.insert(increment);
        } else {
            // 更新递增值
            increment.setValue(increment.getValue() + 1);
            fileSequenceMapper.updateById(increment);
        }
        return increment.getValue();
    }

    //将文件以流的形式一次性读取到内存，通过响应输出流输出到前端
    @Override
    public void download(String filesUUID, HttpServletResponse response) {
        try {

            // 忽略 favicon.ico 请求
            if ("favicon.ico".equals(filesUUID)) {
                // 直接返回，不做任何处理
                return;
            }

            // 确保 filesUUID 是文件名，而不是完整的路径
            // 如果 filesUUID 包含路径分隔符，需要从最后的路径分隔符开始截取文件名
            String fileName = filesUUID.substring(filesUUID.lastIndexOf('/') + 1);

            // 确保路径拼接时使用正确的分隔符
            LocalDateTime now = LocalDateTime.now();
            String yearMonth = now.format(DateTimeFormatter.ofPattern("yyyyMM"));
            File uploadFile = new File(filesUploadPath + "/" + yearMonth, fileName);

            // 检查文件是否存在
            if (!uploadFile.exists()) {
                throw new FileNotFoundException("File not found: " + uploadFile.getAbsolutePath());
            }

            // 设置响应头
            String encodedFileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
            response.addHeader("Content-Disposition", "attachment; filename*=UTF-8''" + encodedFileName);
            response.setContentType("application/octet-stream");

            // 使用 try-with-resources 确保流正确关闭
            try (InputStream inputStream = new BufferedInputStream(new FileInputStream(uploadFile));
                 ServletOutputStream os = response.getOutputStream()) {

                byte[] buffer = new byte[1024];
                int length;
                while ((length = inputStream.read(buffer)) != -1) {
                    os.write(buffer, 0, length);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void deleteFile(String filesUUID) {
        try {

            // 根据文件名查询文件信息
            Files file = getOne(new QueryWrapper<Files>().eq("name", filesUUID));

            if (file == null) {
                throw new RuntimeException("文件不存在，无法删除");
            }

            // 删除本地文件
            String filePath = file.getPath() + "/" + file.getName();
            File localFile = new File(filePath);
            if (localFile.exists()) {
                if (!localFile.delete()) {
                    throw new RuntimeException("文件删除失败");
                }
            }

            // 更新数据库记录
            Files updateFile = new Files();
            updateFile.setId(file.getId());
            updateFile.setFilesName(file.getFilesName());
            updateFile.setName(file.getName());
            updateFile.setPath(file.getPath());
            updateFile.setType(file.getType());
            updateFile.setSize(file.getSize());
            updateFile.setUrl(file.getUrl());
            updateFile.setEnable(false);
            updateFile.setIsDelete(1); // 设置为已删除
            updateFile.setCreateTime(file.getCreateTime()); // 保留原始的 createTime
            updateFile.setUpdateTime(LocalDateTime.now());

            // 提交更新到数据库
            updateById(updateFile); // 调用 updateById 方法更新数据库记录
        } catch (Exception e) {
            throw new RuntimeException("删除文件失败：" + e.getMessage());
        }
    }

    @Override
    public Page<Files> queryFiles(String filesName, LocalDateTime startTime, LocalDateTime endTime, Boolean isDeleted, String fileType, Page<Files> page) {
        QueryWrapper<Files> queryWrapper = new QueryWrapper<>();

        // 默认查询未删除的文件
        if (isDeleted == null) {
            queryWrapper.eq("is_delete", 0);
        } else {
            queryWrapper.eq("is_delete", isDeleted ? 1 : 0);
        }

        if (filesName != null && !filesName.isEmpty()) {
            queryWrapper.like("files_name", filesName);
        }

        if (startTime != null && endTime != null) {
            queryWrapper.ge("create_time", startTime); // 大于等于起始时间
            queryWrapper.le("create_time", endTime);   // 小于等于结束时间
        } else if (startTime != null) {
            queryWrapper.ge("create_time", startTime); // 只有起始时间
        } else if (endTime != null) {
            queryWrapper.le("create_time", endTime);   // 只有结束时间
        }

        if (fileType != null && !fileType.isEmpty()) {
            queryWrapper.eq("type", fileType);
        }

        return page(page, queryWrapper);
    }
}